<?php

declare(strict_types=1);

namespace Dedi\SyliusMarketingPlugin\EventListener;

use Dedi\SyliusMarketingPlugin\Factory\Events\AuthEventFactory;
use Dedi\SyliusMarketingPlugin\Storage\EventsBag;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\EventDispatcher\GenericEvent;
use Symfony\Component\Security\Http\Event\InteractiveLoginEvent;
use Symfony\Contracts\EventDispatcher\Event;

class AuthEventListener implements EventSubscriberInterface
{
    private AuthEventFactory $authEventFactory;
    private EventsBag $eventsStorage;

    public function __construct(AuthEventFactory $authEventFactory, EventsBag $eventsStorage)
    {
        $this->authEventFactory = $authEventFactory;
        $this->eventsStorage = $eventsStorage;
    }
    public static function getSubscribedEvents(): array
    {
        return [
            'sylius.user.security.implicit_login' => 'login',
            'security.interactive_login' => 'loginInteractive',
            'sylius.customer.post_register' => 'signup',
            'sylius.shop_user.post_create' => 'signup',
        ];
    }

    public function signup(GenericEvent $event): void
    {
        $this->eventsStorage->setEvent(
            $this->authEventFactory->signup(),
        );
    }

    //Two different types of events are catch for register redirect and login
    public function login(Event $event): void
    {
        $this->eventsStorage->setEvent(
            $this->authEventFactory->login(),
        );
    }

    public function loginInteractive(InteractiveLoginEvent $event): void
    {
        $this->eventsStorage->setEvent(
            $this->authEventFactory->login(),
        );
    }
}
